#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <sys/stat.h>

#include "tree.h"

struct treenode **find_treenode(struct treenode** parent,int keyyear,int keyweek)
{
    struct treenode * node;
    int result;

    while((node=*parent) !=NULL)
    {
	result=((keyyear*100)+keyweek)-((node->yearno*100)+node->weekno);

	if (result>0)
	    parent=&node->right;
	else if (result <0)
	    parent=&node->left;
	else
	    break;
    }
    return parent;
}

int add_treenode(struct treenode** parent,char * new_entry)
{
    struct treenode ** link;
    struct treenode * node;

    char buff[5];
    int nodeyear;
    int nodeweek;

    /* dont consider hidden files or files with too short a name */
    if((new_entry[0]=='.') || (strlen(new_entry)<11))
	return 1;

    /* get nodes year */
    buff[0]=new_entry[0];
    buff[1]=new_entry[1];
    buff[2]=new_entry[2];
    buff[3]=new_entry[3];
    buff[4]=0;

    nodeyear=atoi(buff);

    if(nodeyear==0)
	return 1;

    /* get nodes week */
    buff[0]=new_entry[10];;
    buff[1]=new_entry[11];
    buff[2]=0;

    nodeweek=atoi(buff);

    if(nodeweek==0)
	return 1;

    link=find_treenode(parent,nodeyear,nodeweek);
    if (*link==NULL)
    {
	node = (struct treenode*)malloc(sizeof(struct treenode));
	/* memory error abort the add */
	if (node==NULL)
	    return 1;
	node->left=node->right=NULL;

	node->yearno=nodeyear;
	node->weekno=nodeweek;

	*link=node;
    }
    else
    {
	/* already in tree - ignore atm */
	return 1;
    }
    return 0;
}

int free_tree_list(struct treenode* node)
{
    if (node->left != NULL)
	free_tree_list(node->left);

    if (node->right != NULL)
	free_tree_list(node->right);

    free(node);

    return 0;
}

struct treenode * make_tree_list(char* path)
{
    struct stat dir_stat;
    struct treenode * treeroot=NULL;

    /* for dir reading */
    DIR *dir_stream;
    struct dirent *dir_entry;

    /* check if peer directory exists */
    if (stat(path,&dir_stat)==-1)
    {
	fprintf(stderr,"Peer directory doesnt exist or is unaccecible\n");
	return NULL;
    }

    if (!S_ISDIR(dir_stat.st_mode))
    {
	fprintf(stderr,"Peer Directory is not a Directory!\n");
	return NULL;
    }

    /* check for files in directory */
    dir_stream=opendir(path);
    if (dir_stream==NULL)
    {
	fprintf(stderr,"Peer Directory open failed\n");
	return NULL;
    }

    dir_entry=readdir(dir_stream);
    while(dir_entry!=NULL)
    {
	add_treenode(&treeroot,dir_entry->d_name);

	dir_entry=readdir(dir_stream);
    }

    return treeroot;
}
